/******************************************************************************
	(C) 2013. IntBuSoft. http://intbusoft.com
	OMarksOpenCV.h - version 1.0		
*******************************************************************************/
#pragma once

#include "OMarks.h"

#include "opencv2/core/core_c.h"
#include "opencv2/imgproc/imgproc_c.h"

#define MAX_CHAR 4000

struct MARKER
{
	int len;
	char result[MAX_CHAR];
	CvPoint Points[4];
};


int FindMarkersOpenCV( IplImage* Image,  int* All_Markers, MARKER* Markers, int Type_Marker, int Param )
{
	int flags = -1;
	if ( Image->nChannels == 1  && Image->depth == 8 ) flags = OMARKS_IMAGE8;
	if ( Image->nChannels == 3  && Image->depth == 8 ) flags = OMARKS_IMAGE24;
	if ( flags == -1 ) return -2;
	int i, j;

	char** Data_Result = new char*[(*All_Markers)];
	int* Size_Data_Result = new int[(*All_Markers)];
	int** Points4_Mark = new int*[(*All_Markers)];
	int all = (*All_Markers); 

	for( i = 0; i < (*All_Markers); i++ )
	{
		Size_Data_Result[i] = MAX_CHAR;	
		Data_Result[i] = new char[MAX_CHAR];
		Points4_Mark[i] = new int[8];
	}
	
	int result = FindMarkers( Image->imageData, Image->width, Image->height,
							  All_Markers, Data_Result, &Size_Data_Result, Points4_Mark, Type_Marker,
							  flags | Param);	 
	for( i = 0; i < (*All_Markers); i++ )
	{
		Markers[i].len = Size_Data_Result[i];
		memcpy( Markers[i].result, Data_Result[i], Markers[i].len );
		Markers[i].result[ MIN(Markers[i].len,MAX_CHAR-1) ] = 0;
		for( j = 0; j < 4; j++ )
		{
			Markers[i].Points[j].x = Points4_Mark[i][j*2];
			Markers[i].Points[j].y = Points4_Mark[i][j*2+1];
		}
	}

	for( i = 0; i < all; i++ )
	{
		delete [] Data_Result[i];
		delete [] Points4_Mark[i];
	}

	delete [] Data_Result;
	delete [] Size_Data_Result ;
	delete [] Points4_Mark;

	return result;
}


#define MAX_MARKERS_GREW		10 

void DMarksProcessedOpenCV(   IplImage* Image, void** Marks, int* All_Markers, MARKER* Markers, int flags )
{
	int flags_ = -1;
	if ( Image->nChannels == 1  && Image->depth == 8 ) flags_ = OMARKS_IMAGE8;
	if ( Image->nChannels == 3  && Image->depth == 8 ) flags_ = OMARKS_IMAGE24;
	if ( flags_ == -1 ) return;

	char** Data_Result = new char*[(*All_Markers)+MAX_MARKERS_GREW];
	int* Size_Data_Result = new int[(*All_Markers)+MAX_MARKERS_GREW];
	int** Points4_Mark = new int*[(*All_Markers)+MAX_MARKERS_GREW];	
	int all = (*All_Markers)+MAX_MARKERS_GREW;

	int i, j;
	for( i = 0; i < (*All_Markers) ; i++ )
	{
		Size_Data_Result[i] = Markers[i].len;	
		Data_Result[i] = new char[Markers[i].len];
		memcpy( Data_Result[i], Markers[i].result, Markers[i].len );
		Points4_Mark[i] = new int[8];
		for( j = 0; j < 4; j++ )
		{
			Points4_Mark[i][j*2] = Markers[i].Points[j].x;
			Points4_Mark[i][j*2+1] = Markers[i].Points[j].y;
		}
	}	
	for( i = (*All_Markers); i < (*All_Markers) + MAX_MARKERS_GREW ; i++ )
	{
		Size_Data_Result[i] = 0;
		Data_Result[i] = new char[MAX_CHAR];		
		Points4_Mark[i] = new int[8];
	}

	DMarksProcessed( Image->imageData, Image->width, Image->height, 
				     Marks,  All_Markers, Data_Result, &Size_Data_Result, Points4_Mark, flags | flags_ );

	for( i = 0; i < (*All_Markers); i++ )
	{
		Markers[i].len = Size_Data_Result[i];
		memcpy( Markers[i].result, Data_Result[i], Markers[i].len );
		Markers[i].result[ MIN(Markers[i].len,MAX_CHAR-1) ] = 0;
		for( j = 0; j < 4; j++ )
		{
			Markers[i].Points[j].x = Points4_Mark[i][j*2];
			Markers[i].Points[j].y = Points4_Mark[i][j*2+1];
		}
	}

	for( i = 0; i < all; i++ )
	{
		delete [] Data_Result[i];
		delete [] Points4_Mark[i];
	}


	delete [] Data_Result;
	delete [] Size_Data_Result ;
	delete [] Points4_Mark;	
}